/*********************************************************************
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2012, Willow Garage, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of Willow Garage nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *********************************************************************/

/* Author: Chittaranjan Srinivas Swaminathan */

#include <chomp_interface/chomp_planning_context.h>
#include <moveit/robot_state/conversions.h>

namespace chomp_interface
{
CHOMPPlanningContext::CHOMPPlanningContext(const std::string& name, const std::string& group,
                                           const moveit::core::RobotModelConstPtr& model,
                                           const rclcpp::Node::SharedPtr& node)
  : planning_interface::PlanningContext(name, group), robot_model_(model)
{
  chomp_interface_ = std::make_shared<CHOMPInterface>(node);
}

void CHOMPPlanningContext::solve(planning_interface::MotionPlanDetailedResponse& res)
{
  chomp_interface_->solve(planning_scene_, request_, chomp_interface_->getParams(), res);
}

void CHOMPPlanningContext::solve(planning_interface::MotionPlanResponse& res)
{
  planning_interface::MotionPlanDetailedResponse res_detailed;
  solve(res_detailed);
  if (res_detailed.error_code.val == moveit_msgs::msg::MoveItErrorCodes::SUCCESS)
  {
    res.trajectory = res_detailed.trajectory[0];
    res.planning_time = res_detailed.processing_time[0];
  }

  res.error_code = res_detailed.error_code;
}

bool CHOMPPlanningContext::terminate()
{
  // TODO - make interruptible
  return true;
}

void CHOMPPlanningContext::clear()
{
}

}  // namespace chomp_interface
